@@ -1,27 +1,23 @@
 # -*- test-case-name: twisted.test.test_tcp -*-
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-
-"""Various asynchronous TCP/IP classes.
+"""
+Various asynchronous TCP/IP classes.
 
 End users shouldn't use this module directly - use the reactor APIs instead.
 
-Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
+Maintainer: Itamar Shtull-Trauring
 """
 
 
 # System Imports
 import os
-import stat
 import types
-import exceptions
 import socket
 import sys
-import select
 import operator
-import warnings
+
 try:
     import fcntl
 except ImportError:
@@ -33,11 +29,12 @@ try:
 except ImportError:
     SSL = None
 
-from twisted.python.runtime import platform, platformType
+from twisted.python.runtime import platformType
+
 
 if platformType == 'win32':
-     # no such thing as WSAEPERM or error code 10001 according to winsock.h or MSDN
-    EPERM=object()
+    # no such thing as WSAEPERM or error code 10001 according to winsock.h or MSDN
+    EPERM = object()
     from errno import WSAEINVAL as EINVAL
     from errno import WSAEWOULDBLOCK as EWOULDBLOCK
     from errno import WSAEINPROGRESS as EINPROGRESS
@@ -47,7 +44,15 @@ if platformType == 'win32':
     from errno import WSAENOTCONN as ENOTCONN
     from errno import WSAEINTR as EINTR
     from errno import WSAENOBUFS as ENOBUFS
-    EAGAIN=EWOULDBLOCK
+    from errno import WSAEMFILE as EMFILE
+    # No such thing as WSAENFILE, either.
+    ENFILE = object()
+    # Nor ENOMEM
+    ENOMEM = object()
+    EAGAIN = EWOULDBLOCK
+    from errno import WSAECONNRESET as ECONNABORTED
+
+    from twisted.python.win32 import formatError as strerror
 else:
     from errno import EPERM
     from errno import EINVAL
@@ -59,20 +64,24 @@ else:
     from errno import ENOTCONN
     from errno import EINTR
     from errno import ENOBUFS
+    from errno import EMFILE
+    from errno import ENFILE
+    from errno import ENOMEM
     from errno import EAGAIN
+    from errno import ECONNABORTED
+
+    from os import strerror
+
+from errno import errorcode
 
 # Twisted Imports
-from twisted.internet import protocol, defer, base, address
-from twisted.persisted import styles
-from twisted.python import log, failure, reflect, components
+from twisted.internet import defer, base, address
+from twisted.python import log, failure, reflect
 from twisted.python.util import unsignedID
 from twisted.internet.error import CannotListenError
+from twisted.internet import abstract, main, interfaces, error
+
 
-# Sibling Imports
-import abstract
-import main
-import interfaces
-import error
 
 class _SocketCloser:
     _socketShutdownMethod = 'shutdown'
@@ -93,17 +102,37 @@ class _SocketCloser:
         except socket.error:
             pass
 
+
+
 class _TLSMixin:
     _socketShutdownMethod = 'sock_shutdown'
 
     writeBlockedOnRead = 0
     readBlockedOnWrite = 0
     _userWantRead = _userWantWrite = True
-    
+
     def getPeerCertificate(self):
         return self.socket.get_peer_certificate()
 
     def doRead(self):
+        if self.disconnected:
+            # See the comment in the similar check in doWrite below.
+            # Additionally, in order for anything other than returning
+            # CONNECTION_DONE here to make sense, it will probably be necessary
+            # to implement a way to switch back to TCP from TLS (actually, if
+            # we did something other than return CONNECTION_DONE, that would be
+            # a big part of implementing that feature).  In other words, the
+            # expectation is that doRead will be called when self.disconnected
+            # is True only when the connection has been lost.  It's possible
+            # that the other end could stop speaking TLS and then send us some
+            # non-TLS data.  We'll end up ignoring that data and dropping the
+            # connection.  There's no unit tests for this check in the cases
+            # where it makes a difference.  The test suite only hits this
+            # codepath when it would have otherwise hit the SSL.ZeroReturnError
+            # exception handler below, which has exactly the same behavior as
+            # this conditional.  Maybe that's the only case that can ever be
+            # triggered, I'm not sure.  -exarkun
+            return main.CONNECTION_DONE
         if self.writeBlockedOnRead:
             self.writeBlockedOnRead = 0
             self._resetReadWrite()
@@ -130,10 +159,21 @@ class _TLSMixin:
     def doWrite(self):
         # Retry disconnecting
         if self.disconnected:
+            # This case is triggered when "disconnected" is set to True by a
+            # call to _postLoseConnection from FileDescriptor.doWrite (to which
+            # we upcall at the end of this overridden version of that API).  It
+            # means that while, as far as any protocol connected to this
+            # transport is concerned, the connection no longer exists, the
+            # connection *does* actually still exist.  Instead of closing the
+            # connection in the overridden _postLoseConnection, we probably
+            # tried (and failed) to send a TLS close alert.  The TCP connection
+            # is still up and we're waiting for the socket to become writeable
+            # enough for the TLS close alert to actually be sendable.  Only
+            # then will the connection actually be torn down. -exarkun
             return self._postLoseConnection()
         if self._writeDisconnected:
             return self._closeWriteConnection()
-        
+
         if self.readBlockedOnWrite:
             self.readBlockedOnWrite = 0
             self._resetReadWrite()
@@ -161,24 +201,35 @@ class _TLSMixin:
         except SSL.Error, e:
             return e
 
+
     def _postLoseConnection(self):
-        """Gets called after loseConnection(), after buffered data is sent.
+        """
+        Gets called after loseConnection(), after buffered data is sent.
 
         We try to send an SSL shutdown alert, but if it doesn't work, retry
         when the socket is writable.
         """
-        self.disconnected=1
+        # Here, set "disconnected" to True to trick higher levels into thinking
+        # the connection is really gone.  It's not, and we're not going to
+        # close it yet.  Instead, we'll try to send a TLS close alert to shut
+        # down the TLS connection cleanly.  Only after we actually get the
+        # close alert into the socket will we disconnect the underlying TCP
+        # connection.
+        self.disconnected = True
         if hasattr(self.socket, 'set_shutdown'):
+            # If possible, mark the state of the TLS connection as having
+            # already received a TLS close alert from the peer.  Why do
+            # this???
             self.socket.set_shutdown(SSL.RECEIVED_SHUTDOWN)
         return self._sendCloseAlert()
 
-    _first=False
+
     def _sendCloseAlert(self):
         # Okay, *THIS* is a bit complicated.
-        
+
         # Basically, the issue is, OpenSSL seems to not actually return
         # errors from SSL_shutdown. Therefore, the only way to
-        # determine if the close notification has been sent is by 
+        # determine if the close notification has been sent is by
         # SSL_shutdown returning "done". However, it will not claim it's
         # done until it's both sent *and* received a shutdown notification.
 
@@ -222,15 +273,28 @@ class _TLSMixin:
             # Note that this is tested for by identity below.
             return main.CONNECTION_DONE
         else:
+            # For some reason, the close alert wasn't sent.  Start writing
+            # again so that we'll get another chance to send it.
             self.startWriting()
+            # On Linux, select will sometimes not report a closed file
+            # descriptor in the write set (in particular, it seems that if a
+            # send() fails with EPIPE, the socket will not appear in the write
+            # set).  The shutdown call above (which calls down to SSL_shutdown)
+            # may have swallowed a write error.  Therefore, also start reading
+            # so that if the socket is closed we will notice.  This doesn't
+            # seem to be a problem for poll (because poll reports errors
+            # separately) or with select on BSD (presumably because, unlike
+            # Linux, it doesn't implement select in terms of poll and then map
+            # POLLHUP to select's in fd_set).
+            self.startReading()
             return None
 
     def _closeWriteConnection(self):
         result = self._sendCloseAlert()
-        
+
         if result is main.CONNECTION_DONE:
             return Connection._closeWriteConnection(self)
-        
+
         return result
 
     def startReading(self):
@@ -260,12 +324,39 @@ class _TLSMixin:
             self.startWriting()
         else:
             self.stopWriting()
-        
+
         if self._userWantRead:
             self.startReading()
         else:
             self.stopReading()
 
+
+
+class _TLSDelayed(object):
+    """
+    State tracking record for TLS startup parameters.  Used to remember how
+    TLS should be started when starting it is delayed to wait for the output
+    buffer to be flushed.
+
+    @ivar bufferedData: A C{list} which contains all the data which was
+        written to the transport after an attempt to start TLS was made but
+        before the buffers outstanding at that time could be flushed and TLS
+        could really be started.  This is appended to by the transport's
+        write and writeSequence methods until it is possible to actually
+        start TLS, then it is written to the TLS-enabled transport.
+
+    @ivar context: An SSL context factory object to use to start TLS.
+
+    @ivar extra: An extra argument to pass to the transport's C{startTLS}
+        method.
+    """
+    def __init__(self, bufferedData, context, extra):
+        self.bufferedData = bufferedData
+        self.context = context
+        self.extra = extra
+
+
+
 def _getTLSClass(klass, _existing={}):
     if klass not in _existing:
         class TLSConnection(_TLSMixin, klass):
@@ -273,11 +364,17 @@ def _getTLSClass(klass, _existing={}):
         _existing[klass] = TLSConnection
     return _existing[klass]
 
+
+
 class Connection(abstract.FileDescriptor, _SocketCloser):
-    """I am the superclass of all socket-based FileDescriptors.
+    """
+    Superclass of all socket-based FileDescriptors.
 
     This is an abstract superclass of all objects which represent a TCP/IP
     connection based socket.
+
+    @ivar logstr: prefix used when logging events related to this connection.
+    @type logstr: C{str}
     """
 
     implements(interfaces.ITCPTransport, interfaces.ISystemHandle)
@@ -292,22 +389,15 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
         self.protocol = protocol
 
     if SSL:
-
-        def startTLS(self, ctx):
+        _tlsWaiting = None
+        def startTLS(self, ctx, extra):
             assert not self.TLS
-            error=False
             if self.dataBuffer or self._tempDataBuffer:
-                self.dataBuffer += "".join(self._tempDataBuffer)
-                self._tempDataBuffer = []
-                self._tempDataLen = 0
-                written = self.writeSomeData(buffer(self.dataBuffer, self.offset))
-                offset = self.offset
-                dataLen = len(self.dataBuffer)
-                self.offset = 0
-                self.dataBuffer = ""
-                if isinstance(written, Exception) or (offset + written != dataLen):
-                    error=True
-
+                # pre-TLS bytes are still being written.  Starting TLS now
+                # will do the wrong thing.  Instead, mark that we're trying
+                # to go into the TLS state.
+                self._tlsWaiting = _TLSDelayed([], ctx, extra)
+                return False
 
             self.stopReading()
             self.stopWriting()
@@ -315,19 +405,44 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
             self.socket = SSL.Connection(ctx.getContext(), self.socket)
             self.fileno = self.socket.fileno
             self.startReading()
-            if error:
-                warnings.warn("startTLS with unwritten buffered data currently doesn't work right. See issue #686. Closing connection.", category=RuntimeWarning, stacklevel=2)
-                self.loseConnection()
-                return
+            return True
+
 
         def _startTLS(self):
             self.TLS = 1
             self.__class__ = _getTLSClass(self.__class__)
 
+
+        def write(self, bytes):
+            if self._tlsWaiting is not None:
+                self._tlsWaiting.bufferedData.append(bytes)
+            else:
+                abstract.FileDescriptor.write(self, bytes)
+
+
+        def writeSequence(self, iovec):
+            if self._tlsWaiting is not None:
+                self._tlsWaiting.bufferedData.extend(iovec)
+            else:
+                abstract.FileDescriptor.writeSequence(self, iovec)
+
+
+        def doWrite(self):
+            result = abstract.FileDescriptor.doWrite(self)
+            if self._tlsWaiting is not None:
+                if not self.dataBuffer and not self._tempDataBuffer:
+                    waiting = self._tlsWaiting
+                    self._tlsWaiting = None
+                    self.startTLS(waiting.context, waiting.extra)
+                    self.writeSequence(waiting.bufferedData)
+            return result
+
+
     def getHandle(self):
         """Return the socket for this connection."""
         return self.socket
-    
+
+
     def doRead(self):
         """Calls self.protocol.dataReceived with all available data.
 
@@ -347,6 +462,7 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
             return main.CONNECTION_DONE
         return self.protocol.dataReceived(data)
 
+
     def writeSomeData(self, data):
         """Connection.writeSomeData(data) -> #of bytes written | CONNECTION_LOST
         This writes as much data as possible to the socket and returns either
@@ -365,6 +481,7 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
             else:
                 return main.CONNECTION_LOST
 
+
     def _closeWriteConnection(self):
         try:
             getattr(self.socket, self._socketShutdownMethod)(1)
@@ -377,7 +494,8 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
             except:
                 f = failure.Failure()
                 log.err()
-                self.connectionLost(f)                
+                self.connectionLost(f)
+
 
     def readConnectionLost(self, reason):
         p = interfaces.IHalfCloseableProtocol(self.protocol, None)
@@ -389,7 +507,7 @@ class Connection(abstract.FileDescriptor, _SocketCloser):
                 self.connectionLost(failure.Failure())
         else:
             self.connectionLost(reason)
-    
+
     def connectionLost(self, reason):
         """See abstract.FileDescriptor.connectionLost().
         """
@@ -441,28 +559,26 @@ class BaseClient(Connection):
             reactor.callLater(0, self.failIfNotConnected, error)
 
     def startTLS(self, ctx, client=1):
-        holder = Connection.startTLS(self, ctx)
-        if client:
-            self.socket.set_connect_state()
-        else:
-            self.socket.set_accept_state()
-        return holder
+        if Connection.startTLS(self, ctx, client):
+            if client:
+                self.socket.set_connect_state()
+            else:
+                self.socket.set_accept_state()
+
 
     def stopConnecting(self):
         """Stop attempt to connect."""
         self.failIfNotConnected(error.UserError())
 
     def failIfNotConnected(self, err):
-        if (self.connected or self.disconnected or 
+        """
+        Generic method called when the attemps to connect failed. It basically
+        cleans everything it can: call connectionFailed, stop read and write,
+        delete socket related members.
+        """
+        if (self.connected or self.disconnected or
             not hasattr(self, "connector")):
             return
-        
-        try:
-            self._closeSocket()
-        except AttributeError:
-            pass
-        else:
-            del self.socket, self.fileno
 
         self.connector.connectionFailed(failure.Failure(err))
         if hasattr(self, "reactor"):
@@ -471,6 +587,13 @@ class BaseClient(Connection):
             self.stopWriting()
             del self.connector
 
+        try:
+            self._closeSocket()
+        except AttributeError:
+            pass
+        else:
+            del self.socket, self.fileno
+
     def createInternetSocket(self):
         """(internal) Create a non-blocking socket using
         self.addressFamily, self.socketType.
@@ -482,7 +605,6 @@ class BaseClient(Connection):
             fcntl.fcntl(s.fileno(), fcntl.F_SETFD, old | fcntl.FD_CLOEXEC)
         return s
 
-
     def resolveAddress(self):
         if abstract.isIPAddress(self.addr[0]):
             self._setRealAddress(self.addr[0])
@@ -504,15 +626,19 @@ class BaseClient(Connection):
             # was scheduled via a callLater in self._finishInit
             return
 
-        # on windows failed connects are reported on exception
-        # list, not write or read list.
-        if platformType == "win32" or sys.platform == "cygwin":
-            r, w, e = select.select([], [], [self.fileno()], 0.0)
-            if e:
-                err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
-                self.failIfNotConnected(error.getConnectError((err, os.strerror(err))))
-                return
+        err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
+        if err:
+            self.failIfNotConnected(error.getConnectError((err, strerror(err))))
+            return
 
+
+        # doConnect gets called twice.  The first time we actually need to
+        # start the connection attempt.  The second time we don't really
+        # want to (SO_ERROR above will have taken care of any errors, and if
+        # it reported none, the mere fact that doConnect was called again is
+        # sufficient to indicate that the connection has succeeded), but it
+        # is not /particularly/ detrimental to do so.  This should get
+        # cleaned up some day, though.
         try:
             connectResult = self.socket.connect_ex(self.realAddress)
         except socket.error, se:
@@ -528,7 +654,7 @@ class BaseClient(Connection):
                 self.startWriting()
                 return
             else:
-                self.failIfNotConnected(error.getConnectError((connectResult, os.strerror(connectResult))))
+                self.failIfNotConnected(error.getConnectError((connectResult, strerror(connectResult))))
                 return
 
         # If I have reached this point without raising or returning, that means
@@ -543,9 +669,9 @@ class BaseClient(Connection):
     def _connectDone(self):
         self.protocol = self.connector.buildProtocol(self.getPeer())
         self.connected = 1
-        self.protocol.makeConnection(self)
-        self.logstr = self.protocol.__class__.__name__+",client"
+        self.logstr = self.protocol.__class__.__name__ + ",client"
         self.startReading()
+        self.protocol.makeConnection(self)
 
     def connectionLost(self, reason):
         if not self.connected:
@@ -592,7 +718,7 @@ class Client(BaseClient):
 
         This indicates the address that I am connected to.
         """
-        return address.IPv4Address('TCP', *(self.addr + ('INET',)))
+        return address.IPv4Address('TCP', *(self.realAddress + ('INET',)))
 
     def __repr__(self):
         s = '<%s to %s at %x>' % (self.__class__, self.addr, unsignedID(self))
@@ -600,26 +726,32 @@ class Client(BaseClient):
 
 
 class Server(Connection):
-    """Serverside socket-stream connection class.
+    """
+    Serverside socket-stream connection class.
 
-    I am a serverside network connection transport; a socket which came from an
-    accept() on a server.
+    This is a serverside network connection transport; a socket which came from
+    an accept() on a server.
     """
 
-    def __init__(self, sock, protocol, client, server, sessionno):
-        """Server(sock, protocol, client, server, sessionno)
+    def __init__(self, sock, protocol, client, server, sessionno, reactor):
+        """
+        Server(sock, protocol, client, server, sessionno)
 
-        Initialize me with a socket, a protocol, a descriptor for my peer (a
+        Initialize it with a socket, a protocol, a descriptor for my peer (a
         tuple of host, port describing the other end of the connection), an
         instance of Port, and a session number.
         """
-        Connection.__init__(self, sock, protocol)
+        Connection.__init__(self, sock, protocol, reactor)
         self.server = server
         self.client = client
         self.sessionno = sessionno
         self.hostname = client[0]
-        self.logstr = "%s,%s,%s" % (self.protocol.__class__.__name__, sessionno, self.hostname)
-        self.repstr = "<%s #%s on %s>" % (self.protocol.__class__.__name__, self.sessionno, self.server.port)
+        self.logstr = "%s,%s,%s" % (self.protocol.__class__.__name__,
+                                    sessionno,
+                                    self.hostname)
+        self.repstr = "<%s #%s on %s>" % (self.protocol.__class__.__name__,
+                                          self.sessionno,
+                                          self.server._realPortNumber)
         self.startReading()
         self.connected = 1
 
@@ -629,12 +761,12 @@ class Server(Connection):
         return self.repstr
 
     def startTLS(self, ctx, server=1):
-        holder = Connection.startTLS(self, ctx)
-        if server:
-            self.socket.set_accept_state()
-        else:
-            self.socket.set_connect_state()
-        return holder
+        if Connection.startTLS(self, ctx, server):
+            if server:
+                self.socket.set_accept_state()
+            else:
+                self.socket.set_connect_state()
+
 
     def getHost(self):
         """Returns an IPv4Address.
@@ -651,15 +783,29 @@ class Server(Connection):
         return address.IPv4Address('TCP', *(self.client + ('INET',)))
 
 class Port(base.BasePort, _SocketCloser):
-    """I am a TCP server port, listening for connections.
+    """
+    A TCP server port, listening for connections.
 
-    When a connection is accepted, I will call my factory's buildProtocol with
-    the incoming connection as an argument, according to the specification
-    described in twisted.internet.interfaces.IProtocolFactory.
+    When a connection is accepted, this will call a factory's buildProtocol
+    with the incoming address as an argument, according to the specification
+    described in L{twisted.internet.interfaces.IProtocolFactory}.
 
-    If you wish to change the sort of transport that will be used, my
-    `transport' attribute will be called with the signature expected for
-    Server.__init__, so it can be replaced.
+    If you wish to change the sort of transport that will be used, the
+    C{transport} attribute will be called with the signature expected for
+    C{Server.__init__}, so it can be replaced.
+
+    @ivar deferred: a deferred created when L{stopListening} is called, and
+        that will fire when connection is lost. This is not to be used it
+        directly: prefer the deferred returned by L{stopListening} instead.
+    @type deferred: L{defer.Deferred}
+
+    @ivar disconnecting: flag indicating that the L{stopListening} method has
+        been called and that no connections should be accepted anymore.
+    @type disconnecting: C{bool}
+
+    @ivar connected: flag set once the listen has successfully been called on
+        the socket.
+    @type connected: C{bool}
     """
 
     implements(interfaces.IListeningPort)
@@ -698,6 +844,7 @@ class Port(base.BasePort, _SocketCloser):
             s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         return s
 
+
     def startListening(self):
         """Create and bind my socket, and begin listening on it.
 
@@ -720,16 +867,18 @@ class Port(base.BasePort, _SocketCloser):
         # can explain it, perhaps we should re-arrange them.
         self.factory.doStart()
         skt.listen(self.backlog)
-        self.connected = 1
+        self.connected = True
         self.socket = skt
         self.fileno = self.socket.fileno
         self.numberAccepts = 100
 
         self.startReading()
 
+
     def _buildAddr(self, (host, port)):
         return address._ServerFactoryIPv4Address('TCP', host, port)
 
+
     def doRead(self):
         """Called when my socket is ready for reading.
 
@@ -755,7 +904,29 @@ class Port(base.BasePort, _SocketCloser):
                         self.numberAccepts = i
                         break
                     elif e.args[0] == EPERM:
+                        # Netfilter on Linux may have rejected the
+                        # connection, but we get told to try to accept()
+                        # anyway.
                         continue
+                    elif e.args[0] in (EMFILE, ENOBUFS, ENFILE, ENOMEM, ECONNABORTED):
+
+                        # Linux gives EMFILE when a process is not allowed
+                        # to allocate any more file descriptors.  *BSD and
+                        # Win32 give (WSA)ENOBUFS.  Linux can also give
+                        # ENFILE if the system is out of inodes, or ENOMEM
+                        # if there is insufficient memory to allocate a new
+                        # dentry.  ECONNABORTED is documented as possible on
+                        # both Linux and Windows, but it is not clear
+                        # whether there are actually any circumstances under
+                        # which it can happen (one might expect it to be
+                        # possible if a client sends a FIN or RST after the
+                        # server sends a SYN|ACK but before application code
+                        # calls accept(2), however at least on Linux this
+                        # _seems_ to be short-circuited by syncookies.
+
+                        log.msg("Could not accept new connection (%s)" % (
+                            errorcode[e.args[0]],))
+                        break
                     raise
 
                 protocol = self.factory.buildProtocol(self._buildAddr(addr))
@@ -764,7 +935,7 @@ class Port(base.BasePort, _SocketCloser):
                     continue
                 s = self.sessionno
                 self.sessionno = s+1
-                transport = self.transport(skt, protocol, addr, self, s)
+                transport = self.transport(skt, protocol, addr, self, s, self.reactor)
                 transport = self._preMakeConnection(transport)
                 protocol.makeConnection(transport)
             else:
@@ -783,13 +954,14 @@ class Port(base.BasePort, _SocketCloser):
         return transport
 
     def loseConnection(self, connDone=failure.Failure(main.CONNECTION_DONE)):
-        """Stop accepting connections on this port.
-
-        This will shut down my socket and call self.connectionLost().
-        It returns a deferred which will fire successfully when the
-        port is actually closed.
         """
-        self.disconnecting = 1
+        Stop accepting connections on this port.
+
+        This will shut down the socket and call self.connectionLost().  It
+        returns a deferred which will fire successfully when the port is
+        actually closed.
+        """
+        self.disconnecting = True
         self.stopReading()
         if self.connected:
             self.deferred = defer.Deferred()
@@ -798,20 +970,37 @@ class Port(base.BasePort, _SocketCloser):
 
     stopListening = loseConnection
 
+
     def connectionLost(self, reason):
-        """Cleans up my socket.
+        """
+        Cleans up the socket.
         """
         log.msg('(Port %s Closed)' % self._realPortNumber)
         self._realPortNumber = None
+        d = None
+        if hasattr(self, "deferred"):
+            d = self.deferred
+            del self.deferred
+
         base.BasePort.connectionLost(self, reason)
-        self.connected = 0
+        self.connected = False
         self._closeSocket()
         del self.socket
         del self.fileno
-        self.factory.doStop()
-        if hasattr(self, "deferred"):
-            self.deferred.callback(None)
-            del self.deferred
+
+        try:
+            self.factory.doStop()
+        except:
+            self.disconnecting = False
+            if d is not None:
+                d.errback(failure.Failure())
+            else:
+                raise
+        else:
+            self.disconnecting = False
+            if d is not None:
+                d.callback(None)
+
 
     def logPrefix(self):
         """Returns the name of my class, to prefix log entries with.
