@@ -2,5 +2,5 @@ from twisted.python import util
 
 util.moduleMovedForSplit('twisted.protocols.pop3', 'twisted.mail.pop3',
                          'POP3 protocol support', 'Mail',
-                         'http://twistedmatrix.com/projects/mail',
+                         'http://twistedmatrix.com/trac/wiki/TwistedMail',
                          globals())
