@@ -68,7 +68,18 @@ randomBytes = getattr(os, 'urandom', _stub_urandom)
 armor = getattr(base64, 'urlsafe_b64encode', _stub_armor)
 
 class InsecurePath(Exception):
-    pass
+    """
+    Error that is raised when the path provided to FilePath is invalid.
+    """
+
+
+
+class LinkError(Exception):
+    """
+    An error with symlinks - either that there are cyclical symlinks or that
+    symlink are not supported on this platform.
+    """
+
 
 
 class UnlistableError(OSError):
@@ -111,9 +122,12 @@ def _secureEnoughString():
     """
     return armor(sha.new(randomBytes(64)).digest())[:16]
 
+
+
 class _PathHelper:
     """
-    Abstract helper class also used by ZipPath; implements certain utility methods.
+    Abstract helper class also used by ZipPath; implements certain utility
+    methods.
     """
 
     def getContent(self):
@@ -176,17 +190,28 @@ class _PathHelper:
             raise UnlistableError(ose)
         return map(self.child, subnames)
 
-    def walk(self):
+    def walk(self, descend=None):
         """
         Yield myself, then each of my children, and each of those children's
-        children in turn.
+        children in turn.  The optional argument C{descend} is a predicate that
+        takes a FilePath, and determines whether or not that FilePath is
+        traversed/descended into.  It will be called with each path for which
+        C{isdir} returns C{True}.  If C{descend} is not specified, all
+        directories will be traversed (including symbolic links which refer to
+        directories).
+
+        @param descend: A one-argument callable that will return True for
+            FilePaths that should be traversed, False otherwise.
 
         @return: a generator yielding FilePath-like objects.
         """
         yield self
-        if self.isdir():
+        if self.isdir() and (descend is None or descend(self)):
             for c in self.children():
-                for subc in c.walk():
+                for subc in c.walk(descend):
+                    if os.path.realpath(self.path).startswith(
+                        os.path.realpath(subc.path)):
+                        raise LinkError("Cycle in file graph.")
                     yield subc
 
     def sibling(self, path):
@@ -359,6 +384,30 @@ class FilePath(_PathHelper):
             if exists(p2):
                 return self.clonePath(p2)
 
+
+    def realpath(self):
+        """
+        Returns the absolute target as a FilePath if self is a link, self
+        otherwise.  The absolute link is the ultimate file or directory the
+        link refers to (for instance, if the link refers to another link, and
+        another...).  If the filesystem does not support symlinks, or
+        if the link is cyclical, raises a LinkError.
+
+        Behaves like L{os.path.realpath} in that it does not resolve link
+        names in the middle (ex. /x/y/z, y is a link to w - realpath on z
+        will return /x/y/z, not /x/w/z).
+
+        @return: FilePath of the target path
+        @raises LinkError: if links are not supported or links are cyclical.
+        """
+        if self.islink():
+            result = os.path.realpath(self.path)
+            if result == self.path:
+                raise LinkError("Cyclical link - will loop forever")
+            return self.clonePath(result)
+        return self
+
+
     def siblingExtension(self, ext):
         return self.clonePath(self.path+ext)
 
@@ -571,7 +620,9 @@ class FilePath(_PathHelper):
 
     def setContent(self, content, ext='.new'):
         sib = self.siblingExtension(ext)
-        sib.open('w').write(content)
+        f = sib.open('w')
+        f.write(content)
+        f.close()
         if platform.isWindows() and exists(self.path):
             os.unlink(self.path)
         os.rename(sib.path, self.path)
@@ -612,7 +663,45 @@ class FilePath(_PathHelper):
 
     _chunkSize = 2 ** 2 ** 2 ** 2
 
-    def copyTo(self, destination):
+
+    def copyTo(self, destination, followLinks=True):
+        """
+        Copies self to destination.
+
+        If self is a directory, this method copies its children (but not
+        itself) recursively to destination - if destination does not exist as a
+        directory, this method creates it.  If destination is a file, an
+        IOError will be raised.
+
+        If self is a file, this method copies it to destination.  If
+        destination is a file, this method overwrites it.  If destination is a
+        directory, an IOError will be raised.
+
+        If self is a link (and followLinks is False), self will be copied
+        over as a new symlink with the same target as returned by os.readlink.
+        That means that if it is absolute, both the old and new symlink will
+        link to the same thing.  If it's relative, then perhaps not (and
+        it's also possible that this relative link will be broken).
+
+        File/directory permissions and ownership will NOT be copied over.
+
+        If followLinks is True, symlinks are followed so that they're treated
+        as their targets.  In other words, if self is a link, the link's target
+        will be copied.  If destination is a link, self will be copied to the
+        destination's target (the actual destination will be destination's
+        target).  Symlinks under self (if self is a directory) will be
+        followed and its target's children be copied recursively.
+
+        If followLinks is False, symlinks will be copied over as symlinks.
+
+        @param destination: the destination (a FilePath) to which self
+            should be copied
+        @param followLinks: whether symlinks in self should be treated as links
+            or as their targets
+        """
+        if self.islink() and not followLinks:
+            os.symlink(os.readlink(self.path), destination.path)
+            return
         # XXX TODO: *thorough* audit and documentation of the exact desired
         # semantics of this code.  Right now the behavior of existent
         # destination symlinks is convenient, and quite possibly correct, but
@@ -622,7 +711,7 @@ class FilePath(_PathHelper):
                 destination.createDirectory()
             for child in self.children():
                 destChild = destination.child(child.basename())
-                child.copyTo(destChild)
+                child.copyTo(destChild, followLinks)
         elif self.isfile():
             writefile = destination.open('w')
             readfile = self.open()
@@ -646,7 +735,24 @@ class FilePath(_PathHelper):
             raise NotImplementedError(
                 "Only copying of files and directories supported")
 
-    def moveTo(self, destination):
+
+    def moveTo(self, destination, followLinks=True):
+        """
+        Move self to destination - basically renaming self to whatever
+        destination is named.  If destination is an already-existing directory,
+        moves all children to destination if destination is empty.  If
+        destination is a non-empty directory, or destination is a file, an
+        OSError will be raised.
+
+        If moving between filesystems, self needs to be copied, and everything
+        that applies to copyTo applies to moveTo.
+
+        @param destination: the destination (a FilePath) to which self
+            should be copied
+        @param followLinks: whether symlinks in self should be treated as links
+            or as their targets (only applicable when moving between
+            filesystems)
+        """
         try:
             os.rename(self.path, destination.path)
             self.restat(False)
@@ -661,12 +767,12 @@ class FilePath(_PathHelper):
 
                 # that means it's time to copy trees of directories!
                 secsib = destination.temporarySibling()
-                self.copyTo(secsib) # slow
-                secsib.moveTo(destination) # visible
+                self.copyTo(secsib, followLinks) # slow
+                secsib.moveTo(destination, followLinks) # visible
 
                 # done creating new stuff.  let's clean me up.
                 mysecsib = self.temporarySibling()
-                self.moveTo(mysecsib) # visible
+                self.moveTo(mysecsib, followLinks) # visible
                 mysecsib.remove() # slow
             else:
                 raise
