@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_log -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -8,16 +8,18 @@ Logging and metrics infrastructure.
 
 from __future__ import division
 
-# System Imports
 import sys
 import time
 import warnings
 from datetime import datetime
 import logging
 
-# Sibling Imports
+from zope.interface import Interface
+
 from twisted.python import util, context, reflect
 
+
+
 class ILogContext:
     """
     Actually, this interface is just a synoym for the dictionary interface,
@@ -26,6 +28,37 @@ class ILogContext:
     I do not inherit from Interface because the world is a cruel place.
     """
 
+
+
+class ILogObserver(Interface):
+    """
+    An observer which can do something with log events.
+    
+    Given that most log observers are actually bound methods, it's okay to not
+    explicitly declare provision of this interface.
+    """
+    def __call__(eventDict):
+        """
+        Log an event.
+
+        @type eventDict: C{dict} with C{str} keys.
+        @param eventDict: A dictionary with arbitrary keys.  However, these
+            keys are often available:
+              - C{message}: A C{tuple} of C{str} containing messages to be
+                logged.
+              - C{system}: A C{str} which indicates the "system" which is
+                generating this event.
+              - C{isError}: A C{bool} indicating whether this event represents
+                an error.
+              - C{failure}: A L{failure.Failure} instance
+              - C{why}: Used as header of the traceback in case of errors.
+              - C{format}: A string format used in place of C{message} to
+                customize the event.  The intent is for the observer to format
+                a message by doing something like C{format % eventDict}.
+        """
+
+
+
 context.setDefault(ILogContext,
                    {"isError": 0,
                     "system": "-"})
@@ -54,12 +87,15 @@ def callWithLogger(logger, func, *args, **kw):
     except:
         err(system=lp)
 
-def showwarning(message, category, filename, lineno, file=None):
+def showwarning(message, category, filename, lineno, file=None, line=None):
     if file is None:
         msg(warning=message, category=reflect.qual(category), filename=filename, lineno=lineno,
             format="%(filename)s:%(lineno)s: %(category)s: %(warning)s")
     else:
-        _oldshowwarning(message, category, filename, lineno, file)
+        if sys.version_info < (2, 6):
+            _oldshowwarning(message, category, filename, lineno, file)
+        else:
+            _oldshowwarning(message, category, filename, lineno, file, line)
 
 _keepErrors = 0
 _keptErrors = []
@@ -148,6 +184,22 @@ def _clearIgnores():
 def err(_stuff=None, _why=None, **kw):
     """
     Write a failure to the log.
+
+    The C{_stuff} and C{_why} parameters use an underscore prefix to lessen
+    the chance of colliding with a keyword argument the application wishes
+    to pass.  It is intended that they be supplied with arguments passed
+    positionally, not by keyword.
+
+    @param _stuff: The failure to log.  If C{_stuff} is C{None} a new
+        L{Failure} will be created from the current exception state.  If
+        C{_stuff} is an C{Exception} instance it will be wrapped in a
+        L{Failure}.
+    @type _stuff: C{NoneType}, C{Exception}, or L{Failure}.
+
+    @param _why: The source of this failure.  This will be logged along with
+        C{_stuff} and should describe the context in which the failure
+        occurred.
+    @type _why: C{str}
     """
     if _stuff is None:
         _stuff = failure.Failure()
@@ -202,8 +254,9 @@ class LogPublisher:
         """
         Add a new observer.
 
-        Observers are callable objects that will be called with each new log
-        message (a dict).
+        @type other: Provider of L{ILogObserver}
+        @param other: A callable object that will be called with each new log
+            message (a dict).
         """
         assert callable(other)
         self.observers.append(other)
@@ -243,9 +296,11 @@ class LogPublisher:
             except UnicodeEncodeError:
                 raise
             except:
-                o = self.observers.pop(i)
+                observer = self.observers[i]
+                self.observers[i] = lambda event: None
                 err(failure.Failure(),
-                    "Log observer %s failed, removing from observer list." % (o,))
+                    "Log observer %s failed." % (observer,))
+                self.observers[i] = observer
 
 
 try:
@@ -363,12 +418,16 @@ class FileLogObserver:
 
         tzOffset = -self.getTimezoneOffset(when)
         when = datetime.utcfromtimestamp(when + tzOffset)
-        tzHour = int(tzOffset / 60 / 60)
-        tzMin = int(tzOffset / 60 % 60)
-        return '%d-%02d-%02d %02d:%02d:%02d%+03d%02d' % (
+        tzHour = abs(int(tzOffset / 60 / 60))
+        tzMin = abs(int(tzOffset / 60 % 60))
+        if tzOffset < 0:
+            tzSign = '-'
+        else:
+            tzSign = '+'
+        return '%d-%02d-%02d %02d:%02d:%02d%s%02d%02d' % (
             when.year, when.month, when.day,
             when.hour, when.minute, when.second,
-            tzHour, tzMin)
+            tzSign, tzHour, tzMin)
 
     def emit(self, eventDict):
         text = textFromEventDict(eventDict)
