@@ -1,8 +1,8 @@
 # -*- test-case-name: twisted.python.test.test_util -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-import os, sys, hmac, errno, new, inspect
+import os, sys, hmac, errno, new, inspect, warnings
 try:
     import pwd, grp
 except ImportError:
@@ -317,7 +317,6 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0,
 
 
 def dict(*a, **k):
-    import warnings
     import __builtin__
     warnings.warn('twisted.python.util.dict is deprecated.  Use __builtin__.dict instead')
     return __builtin__.dict(*a, **k)
@@ -904,6 +903,47 @@ def gidFromString(gidString):
 
 
 
+def runAsEffectiveUser(euid, egid, function, *args, **kwargs):
+    """
+    Run the given function wrapped with seteuid/setegid calls.
+
+    This will try to minimize the number of seteuid/setegid calls, comparing
+    current and wanted permissions
+
+    @param euid: effective UID used to call the function.
+    @type euid: C{int}
+
+    @type egid: effective GID used to call the function.
+    @param egid: C{int}
+
+    @param function: the function run with the specific permission.
+    @type function: any callable
+
+    @param *args: arguments passed to C{function}
+    @param **kwargs: keyword arguments passed to C{function}
+    """
+    uid, gid = os.geteuid(), os.getegid()
+    if uid == euid and gid == egid:
+        return function(*args, **kwargs)
+    else:
+        if uid != 0 and (uid != euid or gid != egid):
+            os.seteuid(0)
+        if gid != egid:
+            os.setegid(egid)
+        if euid != 0 and (euid != uid or gid != egid):
+            os.seteuid(euid)
+        try:
+            return function(*args, **kwargs)
+        finally:
+            if euid != 0 and (uid != euid or gid != egid):
+                os.seteuid(0)
+            if gid != egid:
+                os.setegid(gid)
+            if uid != 0 and (uid != euid or gid != egid):
+                os.seteuid(uid)
+
+
+
 __all__ = [
     "uniquify", "padTo", "getPluginDirs", "addPluginDir", "sibpath",
     "getPassword", "dict", "println", "keyed_md5", "makeStatBar",
@@ -911,5 +951,5 @@ __all__ = [
     "raises", "IntervalDifferential", "FancyStrMixin", "FancyEqMixin",
     "dsu", "switchUID", "SubclassableCStringIO", "moduleMovedForSplit",
     "unsignedID", "mergeFunctionMetadata", "nameToLabel", "uidFromString",
-    "gidFromString",
+    "gidFromString", "runAsEffectiveUser",
 ]
