@@ -13,8 +13,7 @@ import signal
 import StringIO
 import errno
 import gc
-import warnings
-import socket
+import stat
 try:
     import fcntl
 except ImportError:
@@ -23,9 +22,9 @@ except ImportError:
 from zope.interface.verify import verifyObject
 
 from twisted.internet import reactor, protocol, error, interfaces, defer
-from twisted.internet import selectreactor
 from twisted.trial import unittest
 from twisted.python import util, runtime, procutils
+from twisted.python.compat import set
 
 try:
     from twisted.internet import process
@@ -1098,6 +1097,57 @@ class PosixProcessBase:
         return d
 
 
+    def test_errorInProcessEnded(self):
+        """
+        The handler which reaps a process is removed when the process is
+        reaped, even if the protocol's C{processEnded} method raises an
+        exception.
+        """
+        connected = defer.Deferred()
+        ended = defer.Deferred()
+
+        # This script runs until we disconnect its transport.
+        pythonExecutable = sys.executable
+        scriptPath = util.sibpath(__file__, "process_twisted.py")
+
+        class ErrorInProcessEnded(protocol.ProcessProtocol):
+            """
+            A protocol that raises an error in C{processEnded}.
+            """
+            def makeConnection(self, transport):
+                connected.callback(transport)
+
+            def processEnded(self, reason):
+                reactor.callLater(0, ended.callback, None)
+                raise RuntimeError("Deliberate error")
+
+        # Launch the process.
+        reactor.spawnProcess(
+            ErrorInProcessEnded(), pythonExecutable,
+            [pythonExecutable, scriptPath],
+            env=None, path=None)
+
+        pid = []
+        def cbConnected(transport):
+            pid.append(transport.pid)
+            # There's now a reap process handler registered.
+            self.assertIn(transport.pid, process.reapProcessHandlers)
+
+            # Kill the process cleanly, triggering an error in the protocol.
+            transport.loseConnection()
+        connected.addCallback(cbConnected)
+
+        def checkTerminated(ignored):
+            # The exception was logged.
+            excs = self.flushLoggedErrors(RuntimeError)
+            self.assertEqual(len(excs), 1)
+            # The process is no longer scheduled for reaping.
+            self.assertNotIn(pid[0], process.reapProcessHandlers)
+        ended.addCallback(checkTerminated)
+
+        return ended
+
+
 
 class MockOS(object):
     """
@@ -1145,17 +1195,32 @@ class MockOS(object):
 
     @ivar waitChild: if set, subsequent calls to waitpid will return it.
     @type waitChild: C{None} or a tuple
+
+    @ivar euid: the uid returned by the fake C{os.geteuid}
+    @type euid: C{int}
+
+    @ivar egid: the gid returned by the fake C{os.getegid}
+    @type egid: C{int}
+
+    @ivar seteuidCalls: stored results of C{os.seteuid} calls.
+    @type seteuidCalls: C{list}
+
+    @ivar setegidCalls: stored results of C{os.setegid} calls.
+    @type setegidCalls: C{list}
+
+    @ivar path: the path returned by C{os.path.expanduser}.
+    @type path: C{str}
     """
     exited = False
-    O_RDWR = 1
-    O_NOCTTY = 1
-    WNOHANG = 1
     raiseExec = False
     fdio = None
     child = True
     raiseWaitPid = None
     raiseFork = None
     waitChild = None
+    euid = 0
+    egid = 0
+    path = None
 
     def __init__(self):
         """
@@ -1164,6 +1229,13 @@ class MockOS(object):
         self.actions = []
         self.closed = []
         self.pipeCount = 0
+        self.O_RDWR = os.O_RDWR
+        self.O_NOCTTY = os.O_NOCTTY
+        self.WNOHANG = os.WNOHANG
+        self.WEXITSTATUS = os.WEXITSTATUS
+        self.WIFEXITED = os.WIFEXITED
+        self.seteuidCalls = []
+        self.setegidCalls = []
 
 
     def open(self, dev, flags):
@@ -1293,13 +1365,6 @@ class MockOS(object):
         """
 
 
-    def getegid(self):
-        """
-        Override C{os.getegid}. Return a dumb number.
-        """
-        return 1234
-
-
     def getgid(self):
         """
         Override C{os.getgid}. Return a dumb number.
@@ -1307,13 +1372,6 @@ class MockOS(object):
         return 1235
 
 
-    def geteuid(self):
-        """
-        Override C{os.geteuid}. Return a dumb number.
-        """
-        return 1236
-
-
     def getuid(self):
         """
         Override C{os.getuid}. Return a dumb number.
@@ -1356,6 +1414,55 @@ class MockOS(object):
         self.actions.append(('switchuid', uid, gid))
 
 
+    def openpty(self):
+        """
+        Override C{pty.openpty}, returning fake file descriptors.
+        """
+        return -12, -13
+
+
+    def geteuid(self):
+        """
+        Mock C{os.geteuid}, returning C{self.euid} instead.
+        """
+        return self.euid
+
+
+    def getegid(self):
+        """
+        Mock C{os.getegid}, returning C{self.egid} instead.
+        """
+        return self.egid
+
+
+    def seteuid(self, egid):
+        """
+        Mock C{os.seteuid}, store result.
+        """
+        self.seteuidCalls.append(egid)
+
+
+    def setegid(self, egid):
+        """
+        Mock C{os.setegid}, store result.
+        """
+        self.setegidCalls.append(egid)
+
+
+    def expanduser(self, path):
+        """
+        Mock C{os.path.expanduser}.
+        """
+        return self.path
+
+
+    def getpwnam(self, user):
+        """
+        Mock C{pwd.getpwnam}.
+        """
+        return 0, 0, 1, 2
+
+
 
 if process is not None:
     class DumbProcessWriter(process.ProcessWriter):
@@ -1398,42 +1505,35 @@ class MockProcessTestCase(unittest.TestCase):
     """
     Mock a process runner to test forked child code path.
     """
+    if process is None:
+        skip = "twisted.internet.process is never used on Windows"
 
     def setUp(self):
         """
-        Replace L{process} os, fcntl, sys, switchUID modules with the mock
-        class L{MockOS}.
+        Replace L{process} os, fcntl, sys, switchUID, fdesc and pty modules
+        with the mock class L{MockOS}.
         """
         if gc.isenabled():
             self.addCleanup(gc.enable)
         else:
             self.addCleanup(gc.disable)
         self.mockos = MockOS()
-        self.oldos = os
-        self.oldfcntl = fcntl
-        self.oldsys = sys
-        self.oldSwitchUID = util.switchUID
-        self.oldFdesc = process.fdesc
-        process.os = self.mockos
-        process.fcntl = self.mockos
-        process.sys = self.mockos
-        process.switchUID = self.mockos.switchUID
-        process.fdesc = self.mockos
-        process.Process.processReaderFactory = DumbProcessReader
-        process.Process.processWriterFactory = DumbProcessWriter
+        self.mockos.euid = 1236
+        self.mockos.egid = 1234
+        self.patch(process, "os", self.mockos)
+        self.patch(process, "fcntl", self.mockos)
+        self.patch(process, "sys", self.mockos)
+        self.patch(process, "switchUID", self.mockos.switchUID)
+        self.patch(process, "fdesc", self.mockos)
+        self.patch(process.Process, "processReaderFactory", DumbProcessReader)
+        self.patch(process.Process, "processWriterFactory", DumbProcessWriter)
+        self.patch(process, "pty", self.mockos)
 
 
     def tearDown(self):
         """
-        Restore L{process} modules, and reset processes registered for reap.
+        Reset processes registered for reap.
         """
-        process.os = self.oldos
-        process.fcntl = self.oldfcntl
-        process.sys = self.oldsys
-        process.switchUID = self.oldSwitchUID
-        process.fdesc = self.oldFdesc
-        process.Process.processReaderFactory = process.ProcessReader
-        process.Process.processWriterFactory = process.ProcessWriter
         process.reapProcessHandlers = {}
 
 
@@ -1476,7 +1576,7 @@ class MockProcessTestCase(unittest.TestCase):
         reactor.spawnProcess(p, cmd, ['ouch'], env=None,
                              usePTY=False)
         # It should close the first read pipe, and the 2 last write pipes
-        self.assertEqual(self.mockos.closed, [-1, -4, -6])
+        self.assertEqual(set(self.mockos.closed), set([-1, -4, -6]))
         self.assertEquals(self.mockos.actions, [("fork", False), "waitpid"])
 
 
@@ -1556,6 +1656,62 @@ class MockProcessTestCase(unittest.TestCase):
         self.assertFalse(gc.isenabled())
 
 
+    def test_mockForkErrorCloseFDs(self):
+        """
+        When C{os.fork} raises an exception, the file descriptors created
+        before are closed and don't leak.
+        """
+        self._mockWithForkError()
+        self.assertEqual(set(self.mockos.closed), set([-1, -4, -6, -2, -3, -5]))
+
+
+    def test_mockForkErrorGivenFDs(self):
+        """
+        When C{os.forks} raises an exception and that file descriptors have
+        been specified with the C{childFDs} arguments of
+        L{reactor.spawnProcess}, they are not closed.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+            childFDs={0: -10, 1: -11, 2: -13})
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(self.mockos.closed, [])
+
+        # We can also put "r" or "w" to let twisted create the pipes
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+            childFDs={0: "r", 1: -11, 2: -13})
+        self.assertEqual(set(self.mockos.closed), set([-1, -2]))
+
+
+    def test_mockForkErrorClosePTY(self):
+        """
+        When C{os.fork} raises an exception, the file descriptors created by
+        C{pty.openpty} are closed and don't leak, when C{usePTY} is set to
+        C{True}.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          usePTY=True)
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(set(self.mockos.closed), set([-12, -13]))
+
+
+    def test_mockForkErrorPTYGivenFDs(self):
+        """
+        If a tuple is passed to C{usePTY} to specify slave and master file
+        descriptors and that C{os.fork} raises an exception, these file
+        descriptors aren't closed.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          usePTY=(-20, -21, 'foo'))
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(self.mockos.closed, [])
+
+
     def test_mockWithExecError(self):
         """
         Spawn a process but simulate an error during execution in the client
@@ -1708,6 +1864,39 @@ class MockProcessTestCase(unittest.TestCase):
         proc.reapProcess()
 
 
+    def test_mockErrorInPipe(self):
+        """
+        If C{os.pipe} raises an exception after some pipes where created, the
+        created pipes are closed and don't leak.
+        """
+        pipes = [-1, -2, -3, -4]
+        def pipe():
+            try:
+                return pipes.pop(0), pipes.pop(0)
+            except IndexError:
+                raise OSError()
+        self.mockos.pipe = pipe
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None)
+        self.assertEqual(self.mockos.actions, [])
+        self.assertEqual(set(self.mockos.closed), set([-4, -3, -2, -1]))
+
+
+    def test_mockErrorInForkRestoreUID(self):
+        """
+        If C{os.fork} raises an exception and a UID change has been made, the
+        previous UID and GID are restored.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          uid=8080)
+        self.assertEqual(self.mockos.actions,
+            [('setuid', 0), ('setgid', 0), ("fork", False),
+             ('setregid', 1235, 1234), ('setreuid', 1237, 1236)])
+
+
+
 class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
     # add three non-pty test cases
 
@@ -1957,8 +2146,10 @@ class UtilTestCase(unittest.TestCase):
 
     def tearDown(self):
         """
-        Restore the saved PATH setting.
+        Restore the saved PATH setting, and set all created files readable
+        again so that they can be deleted easily.
         """
+        os.chmod(os.path.join(self.bazbar, "executable"), stat.S_IWUSR)
         if self.oldPath is None:
             try:
                 del os.environ['PATH']
@@ -2080,139 +2271,6 @@ class ClosingPipes(unittest.TestCase):
         return d
 
 
-class SystemEventOrderRegressionTests(unittest.TestCase):
-    """
-    Ordering and reentrancy tests for C{reactor.callWhenRunning} and reactor
-    shutdown (see #3146 and #3168).
-    """
-    def setUp(self):
-        """
-        Clear the SIGCHLD handler, if there is one, to ensure an environment
-        like the one which exists prior to a call to L{reactor.run}.
-        """
-        self.originalHandler = signal.signal(signal.SIGCHLD, signal.SIG_DFL)
-        self.processTransports = []
-
-
-    def tearDown(self):
-        """
-        Restore the original SIGCHLD handler and reap processes as long as
-        there seem to be any remaining.
-        """
-        signal.signal(signal.SIGCHLD, signal.SIG_DFL)
-        while self.processTransports:
-            transport = self.processTransports.pop()
-            if transport.pid is not None:
-                os.waitpid(transport.pid, 0)
-        signal.signal(signal.SIGCHLD, self.originalHandler)
-
-
-    def unbuildReactor(self, reactor):
-        """
-        Clean up any resources which may have been allocated for the given
-        reactor by its creation or by a test which used it.
-        """
-        # Chris says:
-        #
-        # XXX This explicit calls to clean up the waker should become obsolete
-        # when bug #3063 is fixed. -radix, 2008-02-29. Fortunately it should
-        # probably cause an error when bug #3063 is fixed, so it should be
-        # removed in the same branch that fixes it.
-        #
-        # -exarkun
-        reactor.removeReader(reactor.waker)
-        reactor.waker.connectionLost(None)
-
-        # Here's an extra thing unrelated to wakers but necessary for
-        # cleaning up after the reactors we make.  -exarkun
-        reactor.disconnectAll()
-
-
-    def buildReactor(self):
-        """
-        Create and return an instance of L{selectreactor.SelectReactor}.
-        """
-        reactor = selectreactor.SelectReactor()
-        self.addCleanup(self.unbuildReactor, reactor)
-        return reactor
-
-
-    def spawnProcess(self, reactor):
-        """
-        Call C{reactor.spawnProcess} with some simple arguments.  Do this here
-        so that code object referenced by the stack frame has a C{co_filename}
-        attribute set to this file so that L{TestCase.assertWarns} can be used.
-        """
-        self.processTransports.append(
-            reactor.spawnProcess(
-                protocol.ProcessProtocol(), sys.executable,
-                [sys.executable, "-c", ""]))
-
-
-    def test_spawnProcessTooEarlyWarns(self):
-        """
-        C{reactor.spawnProcess} emits a warning if it is called before
-        C{reactor.run}.
-
-        If you can figure out a way to make it safe to run
-        C{reactor.spawnProcess} before C{reactor.run}, you may delete the
-        warning and this test.
-        """
-        reactor = self.buildReactor()
-        self.assertWarns(
-            error.PotentialZombieWarning,
-            error.PotentialZombieWarning.MESSAGE, __file__,
-            self.spawnProcess, reactor)
-
-
-    def test_callWhenRunningSpawnProcessWarningFree(self):
-        """
-        L{PotentialZombieWarning} is not emitted when the reactor is run after
-        C{reactor.callWhenRunning(reactor.spawnProcess, ...)} has been called.
-        """
-        events = []
-        self.patch(warnings, 'warn', lambda *a, **kw: events.append(a))
-        reactor = self.buildReactor()
-        reactor.callWhenRunning(self.spawnProcess, reactor)
-        reactor.callWhenRunning(reactor.stop)
-        reactor.run()
-        self.assertFalse(events)
-
-
-    def test_clientConnectionFailedStopsReactor(self):
-        """
-        The reactor can be stopped by a client factory's
-        C{clientConnectionFailed} method.
-
-        This isn't really a process test but it's here for simplicity of
-        implementation and it won't be very long lived.
-        """
-        class Stop(protocol.ClientFactory):
-            def clientConnectionFailed(self, connector, reason):
-                reactor.stop()
-        probe = socket.socket()
-        probe.bind(('', 0))
-        host, port = probe.getsockname()
-        probe.close()
-        reactor = self.buildReactor()
-        reactor.connectTCP(host, port, Stop())
-        reactor.run()
-
-
-    def test_shutdownTriggersRun(self):
-        """
-        C{reactor.run()} does not return until shutdown triggers have all run.
-        """
-        events = []
-        reactor = self.buildReactor()
-        reactor.addSystemEventTrigger(
-            'after', 'shutdown', events.append, "done")
-        reactor.callWhenRunning(reactor.stop)
-        reactor.run()
-        self.assertEqual(events, ["done"])
-
-
-
 skipMessage = "wrong platform or reactor doesn't support IReactorProcess"
 if (runtime.platform.getType() != 'posix') or (not interfaces.IReactorProcess(reactor, None)):
     PosixProcessTestCase.skip = skipMessage
@@ -2232,6 +2290,3 @@ if not interfaces.IReactorProcess(reactor, None):
     ProcessTestCase.skip = skipMessage
     ClosingPipes.skip = skipMessage
 
-if process is None:
-    MockProcessTestCase.skip = skipMessage
-    SystemEventOrderRegressionTests.skip = skipMessage
