@@ -1,21 +1,21 @@
 # -*- test-case-name: twisted.trial.test.test_tests -*-
-#
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Things likely to be used by writers of unit tests.
 
-Maintainer: Jonathan Lange <jml@twistedmatrix.com>
+Maintainer: Jonathan Lange
 """
 
 
-import doctest
-import os, warnings, sys, tempfile, gc
+import doctest, inspect
+import os, warnings, sys, tempfile, gc, types
 from pprint import pformat
 
 from twisted.internet import defer, utils
 from twisted.python import components, failure, log, monkey
+from twisted.python.reflect import qual
 from twisted.python.compat import set
 from twisted.python import deprecate
 from twisted.python.deprecate import getDeprecationWarningString
@@ -102,6 +102,80 @@ def makeTodo(value):
         return Todo(reason=reason, errors=errors)
 
 
+
+class _Warning(object):
+    """
+    A L{_Warning} instance represents one warning emitted through the Python
+    warning system (L{warnings}).  This is used to insulate callers of
+    L{_collectWarnings} from changes to the Python warnings system which might
+    otherwise require changes to the warning objects that function passes to
+    the observer object it accepts.
+
+    @ivar message: The string which was passed as the message parameter to
+        L{warnings.warn}.
+
+    @ivar category: The L{Warning} subclass which was passed as the category
+        parameter to L{warnings.warn}.
+
+    @ivar filename: The name of the file containing the definition of the code
+        object which was C{stacklevel} frames above the call to
+        L{warnings.warn}, where C{stacklevel} is the value of the C{stacklevel}
+        parameter passed to L{warnings.warn}.
+
+    @ivar lineno: The source line associated with the active instruction of the
+        code object object which was C{stacklevel} frames above the call to
+        L{warnings.warn}, where C{stacklevel} is the value of the C{stacklevel}
+        parameter passed to L{warnings.warn}.
+    """
+    def __init__(self, message, category, filename, lineno):
+        self.message = message
+        self.category = category
+        self.filename = filename
+        self.lineno = lineno
+
+
+
+def _collectWarnings(observeWarning, f, *args, **kwargs):
+    """
+    Call C{f} with C{args} positional arguments and C{kwargs} keyword arguments
+    and collect all warnings which are emitted as a result in a list.
+
+    @param observeWarning: A callable which will be invoked with a L{_Warning}
+        instance each time a warning is emitted.
+
+    @return: The return value of C{f(*args, **kwargs)}.
+    """
+    def showWarning(message, category, filename, lineno, file=None, line=None):
+        assert isinstance(message, Warning)
+        observeWarning(_Warning(
+                message.args[0], category, filename, lineno))
+
+    # Disable the per-module cache for every module otherwise if the warning
+    # which the caller is expecting us to collect was already emitted it won't
+    # be re-emitted by the call to f which happens below.
+    for v in sys.modules.itervalues():
+        if v is not None:
+            try:
+                v.__warningregistry__ = None
+            except:
+                # Don't specify a particular exception type to handle in case
+                # some wacky object raises some wacky exception in response to
+                # the setattr attempt.
+                pass
+
+    origFilters = warnings.filters[:]
+    origShow = warnings.showwarning
+    warnings.simplefilter('always')
+    try:
+        warnings.showwarning = showWarning
+        result = f(*args, **kwargs)
+    finally:
+        warnings.filters[:] = origFilters
+        warnings.showwarning = origShow
+    return result
+
+
+
 class _Assertions(pyunit.TestCase, object):
     """
     Replaces many of the built-in TestCase assertions. In general, these
@@ -113,7 +187,8 @@ class _Assertions(pyunit.TestCase, object):
     """
 
     def fail(self, msg=None):
-        """absolutely fails the test, do not pass go, do not collect $200
+        """
+        Absolutely fail the test.  Do not pass go, do not collect $200.
 
         @param msg: the message that will be displayed as the reason for the
         failure
@@ -121,7 +196,8 @@ class _Assertions(pyunit.TestCase, object):
         raise self.failureException(msg)
 
     def failIf(self, condition, msg=None):
-        """fails the test if C{condition} evaluates to False
+        """
+        Fail the test if C{condition} evaluates to True.
 
         @param condition: any object that defines __nonzero__
         """
@@ -131,7 +207,8 @@ class _Assertions(pyunit.TestCase, object):
     assertNot = assertFalse = failUnlessFalse = failIf
 
     def failUnless(self, condition, msg=None):
-        """fails the test if C{condition} evaluates to True
+        """
+        Fail the test if C{condition} evaluates to False.
 
         @param condition: any object that defines __nonzero__
         """
@@ -141,16 +218,18 @@ class _Assertions(pyunit.TestCase, object):
     assert_ = assertTrue = failUnlessTrue = failUnless
 
     def failUnlessRaises(self, exception, f, *args, **kwargs):
-        """fails the test unless calling the function C{f} with the given C{args}
-        and C{kwargs} raises C{exception}. The failure will report the
-        traceback and call stack of the unexpected exception.
+        """
+        Fail the test unless calling the function C{f} with the given
+        C{args} and C{kwargs} raises C{exception}. The failure will report
+        the traceback and call stack of the unexpected exception.
 
         @param exception: exception type that is to be expected
         @param f: the function to call
 
         @return: The raised exception instance, if it is of the given type.
-        @raise self.failureException: Raised if the function call does not raise an exception
-        or if it raises an exception of a different type.
+        @raise self.failureException: Raised if the function call does
+            not raise an exception or if it raises an exception of a
+            different type.
         """
         try:
             result = f(*args, **kwargs)
@@ -185,8 +264,10 @@ class _Assertions(pyunit.TestCase, object):
     assertEqual = assertEquals = failUnlessEquals = failUnlessEqual
 
     def failUnlessIdentical(self, first, second, msg=None):
-        """fail the test if C{first} is not C{second}. This is an
-        obect-identity-equality test, not an object equality (i.e. C{__eq__}) test
+        """
+        Fail the test if C{first} is not C{second}.  This is an
+        obect-identity-equality test, not an object equality
+        (i.e. C{__eq__}) test.
 
         @param msg: if msg is None, then the failure message will be
         '%r is not %r' % (first, second)
@@ -197,8 +278,10 @@ class _Assertions(pyunit.TestCase, object):
     assertIdentical = failUnlessIdentical
 
     def failIfIdentical(self, first, second, msg=None):
-        """fail the test if C{first} is C{second}. This is an
-        obect-identity-equality test, not an object equality (i.e. C{__eq__}) test
+        """
+        Fail the test if C{first} is C{second}.  This is an
+        obect-identity-equality test, not an object equality
+        (i.e. C{__eq__}) test.
 
         @param msg: if msg is None, then the failure message will be
         '%r is %r' % (first, second)
@@ -209,7 +292,8 @@ class _Assertions(pyunit.TestCase, object):
     assertNotIdentical = failIfIdentical
 
     def failIfEqual(self, first, second, msg=None):
-        """fail the test if C{first} == C{second}
+        """
+        Fail the test if C{first} == C{second}.
 
         @param msg: if msg is None, then the failure message will be
         '%r == %r' % (first, second)
@@ -220,7 +304,8 @@ class _Assertions(pyunit.TestCase, object):
     assertNotEqual = assertNotEquals = failIfEquals = failIfEqual
 
     def failUnlessIn(self, containee, container, msg=None):
-        """fail the test if C{containee} is not found in C{container}
+        """
+        Fail the test if C{containee} is not found in C{container}.
 
         @param containee: the value that should be in C{container}
         @param container: a sequence type, or in the case of a mapping type,
@@ -235,7 +320,8 @@ class _Assertions(pyunit.TestCase, object):
     assertIn = failUnlessIn
 
     def failIfIn(self, containee, container, msg=None):
-        """fail the test if C{containee} is found in C{container}
+        """
+        Fail the test if C{containee} is found in C{container}.
 
         @param containee: the value that should not be in C{container}
         @param container: a sequence type, or in the case of a mapping type,
@@ -250,7 +336,8 @@ class _Assertions(pyunit.TestCase, object):
     assertNotIn = failIfIn
 
     def failIfAlmostEqual(self, first, second, places=7, msg=None):
-        """Fail if the two objects are equal as determined by their
+        """
+        Fail if the two objects are equal as determined by their
         difference rounded to the given number of decimal places
         (default 7) and comparing to zero.
 
@@ -268,7 +355,8 @@ class _Assertions(pyunit.TestCase, object):
     failIfAlmostEquals = failIfAlmostEqual
 
     def failUnlessAlmostEqual(self, first, second, places=7, msg=None):
-        """Fail if the two objects are unequal as determined by their
+        """
+        Fail if the two objects are unequal as determined by their
         difference rounded to the given number of decimal places
         (default 7) and comparing to zero.
 
@@ -286,7 +374,8 @@ class _Assertions(pyunit.TestCase, object):
     failUnlessAlmostEquals = failUnlessAlmostEqual
 
     def failUnlessApproximates(self, first, second, tolerance, msg=None):
-        """asserts that C{first} - C{second} > C{tolerance}
+        """
+        Fail if C{first} - C{second} > C{tolerance}
 
         @param msg: if msg is None, then the failure message will be
                     '%r ~== %r' % (first, second)
@@ -297,9 +386,10 @@ class _Assertions(pyunit.TestCase, object):
     assertApproximates = failUnlessApproximates
 
     def failUnlessFailure(self, deferred, *expectedFailures):
-        """Assert that C{deferred} will errback with one of
-        C{expectedFailures}.  Returns the original Deferred with callbacks
-        added. You will need to return this Deferred from your test case.
+        """
+        Fail if C{deferred} does not errback with one of C{expectedFailures}.
+        Returns the original Deferred with callbacks added. You will need
+        to return this Deferred from your test case.
         """
         def _cb(ignore):
             raise self.failureException(
@@ -316,10 +406,16 @@ class _Assertions(pyunit.TestCase, object):
     assertFailure = failUnlessFailure
 
     def failUnlessSubstring(self, substring, astring, msg=None):
+        """
+        Fail if C{substring} does not exist within C{astring}.
+        """
         return self.failUnlessIn(substring, astring, msg)
     assertSubstring = failUnlessSubstring
 
     def failIfSubstring(self, substring, astring, msg=None):
+        """
+        Fail if C{astring} contains C{substring}.
+        """
         return self.failIfIn(substring, astring, msg)
     assertNotSubstring = failIfSubstring
 
@@ -341,30 +437,22 @@ class _Assertions(pyunit.TestCase, object):
         @return: the result of the original function C{f}.
         """
         warningsShown = []
-        def warnExplicit(*args):
-            warningsShown.append(args)
-
-        origExplicit = warnings.warn_explicit
-        try:
-            warnings.warn_explicit = warnExplicit
-            result = f(*args, **kwargs)
-        finally:
-            warnings.warn_explicit = origExplicit
+        result = _collectWarnings(warningsShown.append, f, *args, **kwargs)
 
         if not warningsShown:
             self.fail("No warnings emitted")
         first = warningsShown[0]
         for other in warningsShown[1:]:
-            if other[:2] != first[:2]:
+            if ((other.message, other.category)
+                != (first.message, first.category)):
                 self.fail("Can't handle different warnings")
-        gotMessage, gotCategory, gotFilename, lineno = first[:4]
-        self.assertEqual(gotMessage, message)
-        self.assertIdentical(gotCategory, category)
+        self.assertEqual(first.message, message)
+        self.assertIdentical(first.category, category)
 
         # Use starts with because of .pyc/.pyo issues.
         self.failUnless(
-            filename.startswith(gotFilename),
-            'Warning in %r, expected %r' % (gotFilename, filename))
+            filename.startswith(first.filename),
+            'Warning in %r, expected %r' % (first.filename, filename))
 
         # It would be nice to be able to check the line number as well, but
         # different configurations actually end up reporting different line
@@ -377,8 +465,8 @@ class _Assertions(pyunit.TestCase, object):
 
     def failUnlessIsInstance(self, instance, classOrTuple):
         """
-        Assert that the given instance is of the given class or of one of the
-        given classes.
+        Fail if C{instance} is not an instance of the given class or of
+        one of the given classes.
 
         @param instance: the object to test the type (first argument of the
             C{isinstance} call).
@@ -389,13 +477,12 @@ class _Assertions(pyunit.TestCase, object):
         """
         if not isinstance(instance, classOrTuple):
             self.fail("%r is not an instance of %s" % (instance, classOrTuple))
-
     assertIsInstance = failUnlessIsInstance
 
     def failIfIsInstance(self, instance, classOrTuple):
         """
-        Assert that the given instance is not of the given class or of one of
-        the given classes.
+        Fail if C{instance} is not an instance of the given class or of
+        one of the given classes.
 
         @param instance: the object to test the type (first argument of the
             C{isinstance} call).
@@ -405,14 +492,23 @@ class _Assertions(pyunit.TestCase, object):
         @type classOrTuple: class, type, or tuple.
         """
         if isinstance(instance, classOrTuple):
-            self.fail("%r is not an instance of %s" % (instance, classOrTuple))
-
+            self.fail("%r is an instance of %s" % (instance, classOrTuple))
     assertNotIsInstance = failIfIsInstance
 
 
 class _LogObserver(object):
     """
     Observes the Twisted logs and catches any errors.
+
+    @ivar _errors: A C{list} of L{Failure} instances which were received as
+        error events from the Twisted logging system.
+
+    @ivar _added: A C{int} giving the number of times C{_add} has been called
+        less the number of times C{_remove} has been called; used to only add
+        this observer to the Twisted logging since once, regardless of the
+        number of calls to the add method.
+
+    @ivar _ignored: A C{list} of exception types which will not be recorded.
     """
 
     def __init__(self):
@@ -420,6 +516,7 @@ class _LogObserver(object):
         self._added = 0
         self._ignored = []
 
+
     def _add(self):
         if self._added == 0:
             log.addObserver(self.gotEvent)
@@ -437,18 +534,21 @@ class _LogObserver(object):
             log._ignore = self._oldIE
             log._clearIgnores = self._oldCI
 
+
     def _ignoreErrors(self, *errorTypes):
         """
         Do not store any errors with any of the given types.
         """
         self._ignored.extend(errorTypes)
 
+
     def _clearIgnores(self):
         """
         Stop ignoring any errors we might currently be ignoring.
         """
         self._ignored = []
 
+
     def flushErrors(self, *errorTypes):
         """
         Flush errors from the list of caught errors. If no arguments are
@@ -469,12 +569,14 @@ class _LogObserver(object):
             self._errors = []
         return flushed
 
+
     def getErrors(self):
         """
         Return a list of errors caught by this observer.
         """
         return self._errors
 
+
     def gotEvent(self, event):
         """
         The actual observer method. Called whenever a message is logged.
@@ -488,10 +590,14 @@ class _LogObserver(object):
                 self._errors.append(f)
 
 
+
 _logObserver = _LogObserver()
 
 _wait_is_running = []
 
+_classFixturesDeprecationMessage = (
+    "%(method)s, deprecated since Twisted 8.2.0, was overridden by "
+    "%(class)s.  Use %(replace)s instead.")
 
 class TestCase(_Assertions):
     """
@@ -535,6 +641,13 @@ class TestCase(_Assertions):
     C{errors} is either an exception class or an iterable of exception
     classes, and C{reason} is a string. See L{Todo} or L{makeTodo} for more
     information.
+
+    @ivar _suppressUpDownWarning: Private flag used by tests for C{setUpClass}
+        and C{tearDownClass} to suppress the deprecation warnings for these
+        methods.  This is necessary since the normal warning suppression
+        mechanism does not work for these warnings.  No code should use this
+        flag aside from tests for these methods.  When support for the methods
+        is removed altogether, so should this flag be removed.
     """
 
     implements(itrial.ITestCase)
@@ -569,6 +682,25 @@ class TestCase(_Assertions):
         cls._instancesRun = set()
     _initInstances = classmethod(_initInstances)
 
+    if sys.version_info >= (2, 6):
+        # Override the comparison defined by the base TestCase which considers
+        # instances of the same class with the same _testMethodName to be
+        # equal.  Since trial puts TestCase instances into a set, that
+        # definition of comparison makes it impossible to run the same test
+        # method twice.  Most likely, trial should stop using a set to hold
+        # tests, but until it does, this is necessary on Python 2.6.  Only
+        # __eq__ and __ne__ are required here, not __hash__, since the
+        # inherited __hash__ is compatible with these equality semantics.  A
+        # different __hash__ might be slightly more efficient (by reducing
+        # collisions), but who cares? -exarkun
+        def __eq__(self, other):
+            return self is other
+
+        def __ne__(self, other):
+            return self is not other
+
+
+
     def _isFirst(self):
         return len(self.__class__._instancesRun) == 0
 
@@ -632,10 +764,32 @@ class TestCase(_Assertions):
         return self.run(*args, **kwargs)
 
     def deferSetUpClass(self, result):
+        """
+        Run the per-class set up fixture, C{setUpClass}, for this test case.
+
+        This must be called only once per TestCase subclass, since it will
+        run the fixture unconditionally.
+
+        @type result: L{IReporter} provider
+        @param result: The result which will be used to report any problems
+            encountered in C{setUpClass}.
+
+        @return: A L{Deferred} which will fire with the result of
+            C{setUpClass} or with C{None} if there is no C{setUpClass}
+            defined.
+        """
         if not hasattr(self, 'setUpClass'):
             d = defer.succeed(None)
             d.addCallback(self.deferSetUp, result)
             return d
+        if not getattr(self, '_suppressUpDownWarning', None):
+            warn = deprecate.getWarningMethod()
+            warn(_classFixturesDeprecationMessage % {
+                    'method': 'setUpClass',
+                    'class': qual(self.__class__),
+                    'replace': 'setUp'},
+                category=DeprecationWarning,
+                stacklevel=0)
         d = self._run('setUpClass', result)
         d.addCallbacks(self.deferSetUp, self._ebDeferSetUpClass,
                        callbackArgs=(result,),
@@ -729,6 +883,31 @@ class TestCase(_Assertions):
                 self._passed = False
 
     def deferTearDownClass(self, ignored, result):
+        """
+        Run the per-class tear down fixture, C{tearDownClass}, for this test
+        case.
+
+        This must be called only once per TestCase subclass, since it will
+        run the fixture unconditionally.  This must not be called if there
+        is no C{tearDownClass} method.
+
+        @param ignored: An ignored parameter.
+
+        @type result: L{IReporter} provider
+        @param result: The result which will be used to report any problems
+            encountered in C{tearDownClass}.
+
+        @return: A L{Deferred} which will fire with the result of
+            C{tearDownClass}.
+        """
+        if not getattr(self, '_suppressUpDownWarning', None):
+            warn = deprecate.getWarningMethod()
+            warn(_classFixturesDeprecationMessage % {
+                    'method': 'tearDownClass',
+                    'class': qual(self.__class__),
+                    'replace': 'tearDown'},
+                category=DeprecationWarning,
+                stacklevel=1)
         d = self._run('tearDownClass', result)
         d.addErrback(self._ebTearDownClass, result)
         return d
@@ -820,6 +999,79 @@ class TestCase(_Assertions):
         return self._observer.flushErrors(*errorTypes)
 
 
+    def flushWarnings(self, offendingFunctions=None):
+        """
+        Remove stored warnings from the list of captured warnings and return
+        them.
+
+        @param offendingFunctions: If C{None}, all warnings issued during the
+            currently running test will be flushed.  Otherwise, only warnings
+            which I{point} to a function included in this list will be flushed.
+            All warnings include a filename and source line number; if these
+            parts of a warning point to a source line which is part of a
+            function, then the warning I{points} to that function.
+        @type offendingFunctions: L{NoneType} or L{list} of functions or methods.
+
+        @raise ValueError: If C{offendingFunctions} is not C{None} and includes
+            an object which is not a L{FunctionType} or L{MethodType} instance.
+
+        @raise IOError: If the source file (.py) for one of the functions in
+            C{offendingFunctions} is not available, its lines cannot be
+            determined and L{IOError} will be raised.  (It may be possible to
+            implement this function without requiring source files by using the
+            co_lnotab attribute of code objects.)
+
+        @return: A C{list}, each element of which is a C{dict} giving
+            information about one warning which was flushed by this call.  The
+            keys of each C{dict} are:
+
+                - C{'message'}: The string which was passed as the I{message}
+                  parameter to L{warnings.warn}.
+
+                - C{'category'}: The warning subclass which was passed as the
+                  I{category} parameter to L{warnings.warn}.
+
+                - C{'filename'}: The name of the file containing the definition
+                  of the code object which was C{stacklevel} frames above the
+                  call to L{warnings.warn}, where C{stacklevel} is the value of
+                  the C{stacklevel} parameter passed to L{warnings.warn}.
+
+                - C{'lineno'}: The source line associated with the active
+                  instruction of the code object object which was C{stacklevel}
+                  frames above the call to L{warnings.warn}, where
+                  C{stacklevel} is the value of the C{stacklevel} parameter
+                  passed to L{warnings.warn}.
+        """
+        if offendingFunctions is None:
+            toFlush = self._warnings[:]
+            self._warnings[:] = []
+        else:
+            toFlush = []
+            for w in self._warnings:
+                for aFunction in offendingFunctions:
+                    if not isinstance(aFunction, (
+                            types.FunctionType, types.MethodType)):
+                        raise ValueError("%r is not a function or method" % (
+                                aFunction,))
+                    filename = inspect.getabsfile(aFunction)
+                    if os.path.normcase(filename) != os.path.normcase(w.filename):
+                        continue
+                    lines, start = inspect.getsourcelines(aFunction)
+                    if not (start <= w.lineno <= start + len(lines)):
+                        continue
+                    # The warning points to this function, flush it and move on
+                    # to the next warning.
+                    toFlush.append(w)
+                    break
+            # Remove everything which is being flushed.
+            map(self._warnings.remove, toFlush)
+
+        return [
+            {'message': w.message, 'category': w.category,
+             'filename': w.filename, 'lineno': w.lineno}
+            for w in toFlush]
+
+
     def addCleanup(self, f, *args, **kwargs):
         """
         Add the given function to a list of functions to be called after the
@@ -834,25 +1086,6 @@ class TestCase(_Assertions):
         self._cleanups.append((f, args, kwargs))
 
 
-    def _captureDeprecationWarnings(self, f, *args, **kwargs):
-        """
-        Call C{f} and capture all deprecation warnings.
-        """
-        warnings = []
-        def accumulateDeprecations(message, category, stacklevel):
-            self.assertEqual(DeprecationWarning, category)
-            self.assertEqual(stacklevel, 2)
-            warnings.append(message)
-
-        originalMethod = deprecate.getWarningMethod()
-        deprecate.setWarningMethod(accumulateDeprecations)
-        try:
-            result = f(*args, **kwargs)
-        finally:
-            deprecate.setWarningMethod(originalMethod)
-        return (warnings, result)
-
-
     def callDeprecated(self, version, f, *args, **kwargs):
         """
         Call a function that was deprecated at a specific version.
@@ -861,13 +1094,13 @@ class TestCase(_Assertions):
         @param f: The deprecated function to call.
         @return: Whatever the function returns.
         """
-        warnings, result = self._captureDeprecationWarnings(
-            f, *args, **kwargs)
+        result = f(*args, **kwargs)
+        warningsShown = self.flushWarnings([self.callDeprecated])
 
-        if len(warnings) == 0:
+        if len(warningsShown) == 0:
             self.fail('%r is not deprecated.' % (f,))
 
-        observedWarning = warnings[0]
+        observedWarning = warningsShown[0]['message']
         expectedWarning = getDeprecationWarningString(f, version)
         self.assertEqual(expectedWarning, observedWarning)
 
@@ -943,29 +1176,46 @@ class TestCase(_Assertions):
             result.stopTest(self)
             return
         self._installObserver()
-        self._passed = False
-        first = False
-        if self._shared:
-            first = self._isFirst()
-            self.__class__._instancesRun.add(self)
-        self._deprecateReactor(reactor)
-        try:
-            if first:
-                d = self.deferSetUpClass(result)
-            else:
-                d = self.deferSetUp(None, result)
+
+        # All the code inside runThunk will be run such that warnings emitted
+        # by it will be collected and retrievable by flushWarnings.
+        def runThunk():
+            self._passed = False
+            first = False
+            if self._shared:
+                first = self._isFirst()
+                self.__class__._instancesRun.add(self)
+            self._deprecateReactor(reactor)
             try:
-                self._wait(d)
+                if first:
+                    d = self.deferSetUpClass(result)
+                else:
+                    d = self.deferSetUp(None, result)
+                try:
+                    self._wait(d)
+                finally:
+                    self._cleanUp(result)
+                    if self._shared and self._isLast():
+                        self._initInstances()
+                        self._classCleanUp(result)
+                    if not self._shared:
+                        self._classCleanUp(result)
             finally:
-                self._cleanUp(result)
-                result.stopTest(self)
-                if self._shared and self._isLast():
-                    self._initInstances()
-                    self._classCleanUp(result)
-                if not self._shared:
-                    self._classCleanUp(result)
-        finally:
-            self._undeprecateReactor(reactor)
+                self._undeprecateReactor(reactor)
+
+        self._warnings = []
+        _collectWarnings(self._warnings.append, runThunk)
+
+        # Any collected warnings which the test method didn't flush get
+        # re-emitted so they'll be logged or show up on stdout or whatever.
+        for w in self.flushWarnings():
+            try:
+                warnings.warn_explicit(**w)
+            except:
+                result.addError(self, failure.Failure())
+
+        result.stopTest(self)
+
 
     def _getReason(self, f):
         if len(f.value.args) > 0:
@@ -1451,4 +1701,3 @@ for methodName in _assertions:
 
 
 __all__ = ['TestCase', 'wait', 'FailTest', 'SkipTest']
-
